@@ -163,7 +163,7 @@ class Camera(BaseCamera):
         if (config.CONFIG["histogram"]!=None) or \
            (config.CONFIG["histogram"]==True):
                 print("i... HISTOGRAM ON ===> MANUAL EGM")
-                set_gem(cc, "def","def","def")
+                set_gem(cc, "def","auto","def") # exp 'def' is different from auto
         else:
             # I CALL SET_GAM from
             set_gem(cc, config.CONFIG['gain'],
@@ -497,7 +497,7 @@ class Camera(BaseCamera):
                         senh.setbox("MODE DM", senh.MODE) #---push UP to avoid DetMot
                         senh.detmo( average, blur)
                         senh.chk_threshold( threshold )
-                        if senh.motion_detected:
+                        if senh.motion_detected: # saving avi on mation detect
                             # print("D... sav mot", senh.motion_detected)
                             senh.save_avi( seconds = -1, name = "dm" )
                     else:
