@@ -66,6 +66,7 @@ class Stream_Enhancer:
 
     avi_last = dt.datetime.now() - dt.timedelta(seconds=90000)
     aviopened_laps = False
+    avi_started = dt.datetime.now()  # I like to have one file per day.....
 
     # -- age of frame for speed
     camera_age = 0
@@ -405,7 +406,7 @@ class Stream_Enhancer:
 
 
 
-    def crosson(self,  dix, diy):
+    def crosson(self,  dix, diy, color = "g"):
         RADIUS=63
         y = int(self.frame.shape[0]/2)
         x = int(self.frame.shape[1]/2)
@@ -413,14 +414,43 @@ class Stream_Enhancer:
         ix = x+dix
         iy = y+diy
 
-        i2=cv2.circle( self.frame, (ix,iy), RADIUS, (0,255,55), 1)
-        i2=cv2.line(i2, (ix-RADIUS+5,iy), (ix-5,iy), (0, 255, 55), thickness=1, lineType=8)
-        i2=cv2.line(i2, (ix+RADIUS-5,iy), (ix+5,iy), (0, 255, 55), thickness=1, lineType=8)
+        if color=="g":
+            lcolor=(0,255,55)
+        elif (color=="r"):
+            lcolor=(55,0,255) #BGR
+        else:
+            lcolor=(0,255,55)
 
-        i2=cv2.line(i2, (ix,iy-RADIUS+5), (ix,iy-5), (0, 255, 55), thickness=1, lineType=8)
-        i2=cv2.line(i2, (ix,iy+RADIUS-5), (ix,iy+5), (0, 255, 55), thickness=1, lineType=8)
+        crscal = 4
+        crnx,crny = int(64/crscal),int(48/crscal)
+        if color=="r":
+            midskip = crnx
+            midskipy = crny
+        else:
+            midskip = 7
+            midskipy = 7
+
+
+        i2=cv2.circle( self.frame, (ix,iy), RADIUS, lcolor, 1)
+        i2=cv2.line(i2, (ix-RADIUS+midskip,iy), (ix-midskip,iy), lcolor, thickness=1, lineType=8)
+        i2=cv2.line(i2, (ix+RADIUS-midskip,iy), (ix+midskip,iy), lcolor, thickness=1, lineType=8)
+
+        i2=cv2.line(i2, (ix,iy-RADIUS+midskipy), (ix,iy-midskipy), lcolor, thickness=1, lineType=8)
+        i2=cv2.line(i2, (ix,iy+RADIUS-midskipy), (ix,iy+midskipy), lcolor, thickness=1, lineType=8)
+
+        # mid
+        i2=cv2.line(i2, (ix,iy), (ix,iy), lcolor, thickness=1, lineType=8)
+
+        if color=="r":
+            #corners  #  position 0.5deg from 11 deg. OK
+            crscal = 4
+            crnx,crny = int(64/crscal),int(48/crscal)
+
+            i2=cv2.line(i2, (ix-crnx,iy-crny), (ix+crnx,iy-crny), lcolor, thickness=1, lineType=8)
+            i2=cv2.line(i2, (ix+crnx,iy-crny), (ix+crnx,iy+crny), lcolor, thickness=1, lineType=8)
+            i2=cv2.line(i2, (ix+crnx,iy+crny), (ix-crnx,iy+crny), lcolor, thickness=1, lineType=8)
+            i2=cv2.line(i2, (ix-crnx,iy+crny), (ix-crnx,iy-crny), lcolor, thickness=1, lineType=8)
 
-        i2=cv2.line(i2, (ix,iy), (ix,iy), (0, 255, 55), thickness=1, lineType=8)
         #return self.frame
 
 
@@ -706,7 +736,7 @@ class Stream_Enhancer:
         # -------------- one shot
         if seconds == 0:
             return
-        elif seconds<0:
+        elif seconds<0: # - means saving avi - motion detect or in uni...
             if not self.aviopened:
                 filename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
                 me = socket.gethostname()
@@ -721,6 +751,22 @@ class Stream_Enhancer:
                 self.aviout = cv2.VideoWriter( filename , fourcc , 25.0, (640,480))
                 self.aviopened = True
 
+            elif (self.aviopened) and ( (dt.datetime.now()-self.avi_started).total_seconds()>86400): # it is opened but too long, one day....
+                self.aviout.release()
+                filename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
+                me = socket.gethostname()
+                filename = f"{me}{name}_{filename}.avi"
+                filename = os.path.expanduser("~/"+filename)
+                print(filename)
+                print(filename)
+                print(filename)
+                print(filename)
+                self.aviopened = True
+                self.avi_started = dt.datetime.now()  #
+
+                fourcc = cv2.VideoWriter_fourcc(*'XVID')
+                self.aviout = cv2.VideoWriter( filename , fourcc , 25.0, (640,480))
+
             self.aviout.write(self.frame)
         # -------------- timelapse if seconds>0:
         elif seconds>0:
