@@ -17,7 +17,7 @@ import cv2
 import datetime
 import time
 
-import datetime as dt
+# import datetime as dt
 
 import os
 
@@ -46,11 +46,11 @@ global FILE_USERPASS
 FILE_USERPASS = "~/.flashcam_upw"
 
 @contextmanager
-def timeout(time):
+def timeout(atime):
     # register a function to raise a TimeoutError on the signal
     signal.signal(signal.SIGALRM, raise_timeout)
     # schedule the signal to be sent after 'time'
-    signal.alarm(time)
+    signal.alarm(atime)
     #print("D... timeout registerred")
 
     try:
@@ -87,136 +87,13 @@ def img_estim(img, thrshld=127):
     # 40 -> 2.2
 
 #------------------------------------------------------------------------ 3
-
-def get_stream3( videodev, passfile ):
-    # localuser
-    u,p=getpass.getuser(),"a"
-    try:
-        with open( os.path.expanduser(passfile) ) as f:
-            print("YES---> PASSWORD FILE  ", passfile )
-            w1 = f.readlines()
-            u= w1[0].strip()
-            p= w1[1].strip()
-    except:
-        print("NO PASSWORD FILE (gs3)  ", passfile)
-        sys.exit(0)
-
-    print("D... capturing from /{}/".format(videodev) )
-    #cam = cv2.VideoCapture( videodev )
-    #stream = urllib.request.urlopen( videodev )
-
-    request = urllib.request.Request( videodev )
-    print("D... USER/PASS", u,p)
-    base64string =base64.b64encode( bytes(  '%s:%s' % (u, p ), 'ascii') )
-    print("D... stream ok1", base64string)
-    request.add_header("Authorization", "Basic %s" % base64string.decode('utf-8'))
-
-    #request.add_header("Authorization", "Basic %s" % base64string)
-    print("D... stream ok2 - request.urlopen (disp)")
-    ok = False
-    stream = None
-    try:
-        stream = urllib.request.urlopen(request, timeout=5)
-        ok = True
-        print("D... stream ok3")
-    except urllib.error.HTTPError as e:
-        print("Server Offline1 ",e)
-        #do stuff here
-    except urllib.error.URLError as e:
-        print("Server Offline2 ",e)
-        #do stuff here
-    except:
-        ok = False
-        stream = None
-        print("X.... Stream from urllib Timeouted OR a bad password")
-    return stream, u, p
-
-
-
-
-
-def display3(videodev, save = False,
-         passfile="~/.pycamfw_userpass"):
-    stream = None
-    stream_length = 1024*50
-    if stream is None: # LOGIN
-        stream, u, p = get_stream3(videodev, passfile)
-
-
-    #print(stream)
-
-    ret_val=0
-    oi = 0
-    oi_frame = 0
-    io_none = 0
-    bytex = b''
-    while ret_val == 0:
-        oi+= 1
-        #with timeout(2):
-        got_data = False
-        frameok = False
-
-        try:
-            # THIS CAN TIMEOUT ######################################### i catch
-            bytex += stream.read(stream_length) # i multiply by 10 as later...
-            got_data = True
-        except:
-            print("X... exception - timout in 1.st stream.read, ")
-            #bytex+=b"\x00\x00\x00"
-            #bytex = b""
-
-        a = bytex.find(b'\xff\xd8') #frame starting
-        b = bytex.find(b'\xff\xd9') #frame ending
-
-        #print(f" {len(bytex):7d} {got_data}   {a:8} {b:8}    {b+2-a}")
-
-        if a != -1 and b != -1:
-            io_none = 0
-            jpg = bytex[a:b+2]
-            bytex = bytex[a+2:]
-            bytex = bytex[b+2:]
-            frameok = False
-            try:
-                frame = cv2.imdecode(np.frombuffer(jpg, dtype=np.uint8), cv2.IMREAD_COLOR)
-                #ret_val= 1
-                io_none = 0
-                frameok = True
-            except:
-                ret_val = 0
-
-        else:
-            ret_val = 0
-            #print("D...                        frame set None http",oi,len(bytex), end="\r")
-            #bytex+=b""
-            #time.sleep(0.1)
-            io_none+=1
-            if io_none>30: #max size of jpg
-                stream = None
-                print("X... ---------------   frame not seen too long:", io_none, "... breaking")
-                io_none = 0
-                break
-
-        if (oi_frame % 20) ==0:
-            print("_"*70)
-            print(f"       #     d_size  got      a      b       jpglen frame   f#" )
-            print("_"*70)
-
-        print(f"{oi:8d}  {len(bytex):7d} {got_data}   {a:8} {b:8}    {b+2-a}  {frameok}  {oi_frame}", end = "\n" if frameok else "\r" , flush=True)
-
-        if frameok:
-            oi_frame+= 1
-
-            cv2.imshow( videodev, frame)
-            key = cv2.waitKey(1)
-
-
 # ================================================================================================
 
 # ================================================================================================
 
 def display2(videodev, save = False,
              passfile="~/.pycamfw_userpass",
-             rotate = 0, vof = 99):
+             rotate = 0, vof = "99,2"):
     """
     """
     #sname,sfilenamea,sme,sfilenamea,sfilenamea,sfourcc,saviout
@@ -225,7 +102,10 @@ def display2(videodev, save = False,
     #frame = None
     global centerX1, centerY1, clicked
     global FILE_USERPASS
-    centerX1, centerY1, clicked = 0,0, False
+    centerX1, centerY1, clicked = 0,0, True # center zoom ion start
+
+
+    filesource = False
 
     def MOUSE(event, x, y, flags, param):
         global centerX1,centerY1, clicked
@@ -249,7 +129,7 @@ def display2(videodev, save = False,
         sname = sname.replace("5000/video","")
         sname = sname.replace("8000/video","")
 
-        sfilenamea = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
+        sfilenamea = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
         sme = socket.gethostname()
         sfilenamea = f"{sme}_{sname}_{sfilenamea}.avi"
         sfilenamea = os.path.expanduser("~/" + sfilenamea)
@@ -290,7 +170,7 @@ def display2(videodev, save = False,
             print("NO PASSWORD FILE (gs) ")
 
 
-        print("D... capturing from web:/{}/".format(videodev) )
+        print("D... capturing from: /{}/".format(videodev) )
         #cam = cv2.VideoCapture( videodev )
         #stream = urllib.request.urlopen( videodev )
 
@@ -306,6 +186,7 @@ def display2(videodev, save = False,
         try:
             stream = urllib.request.urlopen(request, timeout=3) # timeout to 7 from 5 sec.
             ok = True
+            filesource = True
             print("D... stream ok3")
         except urllib.error.HTTPError as e:
             print("Server Offline1? ",e)
@@ -346,6 +227,7 @@ def display2(videodev, save = False,
 
         bytex = b'' # stream
         rpi_name = videodev
+        frame_num = 0
 
         if (str(videodev).find("http://")==0) or (str(videodev).find("https://")==0):
             # infinite loop for stream authentication
@@ -375,7 +257,7 @@ def display2(videodev, save = False,
                 stream, u, p  = get_stream()
         else:
             print("X... use http:// address")
-            sys.exit(0)
+            #sys.exit(0)
 
 
 
@@ -427,8 +309,19 @@ def display2(videodev, save = False,
                         break
 
                     #frame = None
-        if stream is None:
-            continue
+        if 'stream' in locals():
+            if stream is None:
+                continue
+        else:                     # from file ---- 1st moment open
+            ret_val = 0
+            stream = cv2.VideoCapture( videodev )
+            filesource = True
+            print ("X... open  = ",stream.isOpened())
+            ok,frame = stream.read()
+            frame_num+=1
+            # pause = True too early.... defined later
+            print(f"X... {ok}, frame from {videodev}")
+
 #----------------------------------------------------------------
 
         first = True
@@ -450,7 +343,7 @@ def display2(videodev, save = False,
         # ---- bytes per second.  strange looks like 7MB/s
         BPS = 0
         BPSlast=0
-        BPStag = dt.datetime.now()
+        BPStag = datetime.datetime.now()
         FPS = 0
         FPSlast = 0
         frames_total = 0
@@ -460,29 +353,51 @@ def display2(videodev, save = False,
         saved_jpg = False
 
         zoomme=1
+        centerX1, centerY1 = 320,240
         expande=1
         rorate180 = False
 
 
         # measurements (distance)
         measure = 0 #1.7
-        measure_fov = vof # config.CONFIG['vof'] #
 
-        tracker = None
+        if str(vof).find(",")>0:
+            vof = str(vof).strip("(")
+            vof = str(vof).strip(")")
+            measure_fov,measure = float(vof.split(",")[0]),float(vof.split(",")[1])
+        else:
+            measure_fov = float(vof)     # config.CONFIG['vof'] #
+
+
+        cross = None
+        print(" ... ... reset of all trackers/zoom/measure etc..")
+
+        tracker1 = None
         tracker2 = None
-        orb = None
+        tracker1_fname = None # change filename for tracking
+        tracker2_fname = None # change filename for tracking
+        tracker_list = []
+        cropped = None # created during tracking1
+        orb = None  # i dont use
 
-        # -see the values sent from the webpy
-        webframen = "-------" # frame number from web.py()
+        # file - pause
+        pause = True # FOR FILE but not for CAM
+        if filesource==False:
+            pause=False
+        frame_from_file = None # backup the frame:  for effects; also for CAMERA now!
+
+        # -see the values sent from the webpy - i can use in track, but not in savejpg,saveavi!
+        webframen = "" # frame number from web.py()
         webframetime = ""
 
         while connection_ok: #========================================================
             # read the frame from the camera and send it to the server
             #time.sleep(0.05)
 
-            print("-", end="")
             #while True:
             if (str(videodev).find("http://")==0) or (str(videodev).find("https://")==0):
+
+                print("-", end="")
                 artdelay = 0
                 ret_val = 0
                 try:
@@ -515,7 +430,15 @@ def display2(videodev, save = False,
                                     # just a test.... if I can append
                                     #jpg = jpg+b'#FRAME_ACQUISITION_TIME#'+f"a".encode("utf8")
                                     frame = cv2.imdecode(np.frombuffer(jpg, dtype=np.uint8), cv2.IMREAD_COLOR)
-                                    frame_num+=1
+                                    # ----taken for pause
+                                    if (not pause) or  (frame_from_file is None):
+                                        frame_from_file = frame
+                                        frame_num+=1
+                                    else:
+                                        frame = frame_from_file
+                                        #ret_val = 1
+                                        #frame_num+=1 # ??? I keep this stopped or better not?
+
                                     #stream_length = b+2-a
                                     stream_length = int( (b+2-a)*0.9 ) # expected length
 
@@ -527,7 +450,7 @@ def display2(videodev, save = False,
                                         webframen,
                                         BPSlast*8/1024,
                                         FPSlast,
-                                        str(datetime.datetime.now())[11:-5],
+                                        str(datetime.datetime.now())[11:-4],
                                         webframetime[11:],
                                         sfilenamea.replace('/home/','')
                                     ) , end="\r")
@@ -546,8 +469,8 @@ def display2(videodev, save = False,
 
 
                 #print("-2", flush=True,end="")
-                if (dt.datetime.now()-BPStag).total_seconds()>1:
-                    BPStag = dt.datetime.now()
+                if (datetime.datetime.now()-BPStag).total_seconds()>1:
+                    BPStag = datetime.datetime.now()
                     BPSlast=BPS
                     BPS=0
                     FPSlast = FPS
@@ -556,7 +479,16 @@ def display2(videodev, save = False,
 
                 #while
 
-
+            else: #                     from file 2nd point
+                if (not pause) or  (frame_from_file is None):
+                    ret_val,frame = stream.read()
+                    frame_from_file = frame
+                    frame_num+=1
+                else:
+                    frame = frame_from_file
+                    ret_val = 1
+                if ret_val==0:
+                    sys.exit(0)
             if connection_ok:
                 if (ret_val == 0) or (type(frame)=="NoneType"):
                     print("Not a good frame", type(frame), end="\r")
@@ -565,13 +497,18 @@ def display2(videodev, save = False,
                 (w, h, c) = frame.shape
                 frame_area = w*h
                 motion_det = False
-                print(".", end="")
+
+                # print(".", end="")
                 #print("RPINAME=",rpi_name)
                 #print(frame)
 
                 wname = videodev
 
                 frames_total+= 1
+
+                #======================================== GAMES WITH FRAMES
+                #   hack-expand;  tracker; zoom; rotate; save; measure
+
                 # cv2.namedWindow( wname, cv2.WINDOW_KEEPRATIO ,cv2.WINDOW_GUI_EXPANDED)
                 cv2.namedWindow( wname , cv2.WINDOW_KEEPRATIO ) # 2 may allow resize on gigavg
                 # cv2.namedWindow( wname , 2 ) # 2 may allow resize on gigavg
@@ -601,26 +538,53 @@ def display2(videodev, save = False,
                 #frame = cv2.blur( frame, (6,6) ) # doesnt help
 
                 # ======================== track before zoom
-                if tracker!=None:
-                    #print("tracking",tracker)
-                    ok,bbox=tracker.update(frame)
+                if (tracker1!=None) and (not pause):
+                    #print("tracking",tracker1)
+                    ok,bbox=tracker1.update(frame)
                     if not ok:
                         continue
                     bbox = [ round(i*10)/10 for i in bbox]
                     (x,y,w,h)=[v for v in bbox]
                     cx,cy = round( 10*(x+w/2))/10, round(10*(y+h/2))/10
                     #print("tracking",ok,bbox," ->", cx,cy)
-                    with open("tracking1.dat", "a" ) as f:
-                        f.write( f"{webframetime[11:]} {webframen} {cx} {cy}\n" )
+                    with open( tracker1_fname , "a" ) as f:
+                        f.write( f"{webframetime} {webframen} {cx} {cy}\n" )
+                        #   f.write( f"{webframetime[11:]} {webframen} {cx} {cy}\n" )
+
+                    if (webframen.strip(" ")==""):
+                        webframen=frame_num
+                        ttime = int(webframen)
+                    else:
+                        # better this, fractions are kept...
+                        ttime  = datetime.datetime.strptime(webframetime,
+                                                        "%Y-%m-%d %H:%M:%S.%f")
+
+
+                    tracker_list.append( (round(cx), round(cy),  ttime ) )
+                    colmax=255
+                    colmaxb=0
+                    for i in reversed(tracker_list):
+                        x2,y2,ttime=i
+                        frame[ y2,x2 ] = (colmaxb,255-colmax-colmaxb,255)
+                        if colmax>1:
+                            colmax-=1
+                        elif (colmaxb<255) and (colmaxb>1):
+                            colmaxb+=1
+
                     # ------------ play on cropping -- may further stabilize
                     cropped = frame[round(y):round(y+h), round(x):round(x+w)]
-                    resu = np.zeros((640,480))
-                    cropped = cv2.normalize(cropped, resu,0,255,cv2.NORM_MINMAX)
-                    #cropped = cv2.cvtColor(cropped,cv2.COLOR_BGR2GRAY)
-                    #cropped = cv2.blur( cropped, (2,2) )
-                    #(T,cropped) = cv2.threshold(cropped,  100, 255,  cv2.THRESH_BINARY | cv2.THRESH_OTSU)
-                    #cropped = cv2.merge([cropped,cropped,cropped] )
-                    frame[round(y):round(y+h), round(x):round(x+w)] = cropped
+                    # normalize region-   problems/crashes on pause
+                    #resu = np.zeros((640,480))
+                    #cropped = cv2.normalize(cropped, resu,0,255,cv2.NORM_MINMAX)
+                    #    gray
+                    ##cropped = cv2.cvtColor(cropped,cv2.COLOR_BGR2GRAY)
+                    ##cropped = cv2.blur( cropped, (2,2) )
+                    ##(T,cropped) = cv2.threshold(cropped,  100, 255,  cv2.THRESH_BINARY | cv2.THRESH_OTSU)
+                    # merge back to BGR
+                    ##cropped = cv2.merge([cropped,cropped,cropped] )
+
+                    # # return cropped to main image -------------- problem on pause+normalize
+                    #frame[round(y):round(y+h), round(x):round(x+w)] = cropped
 
 
                     # ---ORB feature matching...
@@ -663,10 +627,25 @@ def display2(videodev, save = False,
                     cv2.line(frame,(int(cx2),int(cy2)),
                              ( int(cx2),int(cy2)
                              ),(0,255,255),2,1)
-                    with open("tracking2.dat", "a" ) as f:
+                    with open( tracker2_fname, "a" ) as f:
+                        if webframen=="":
+                            webframen=frame_num
                         f.write( f"{webframetime} {webframen} {cx2} {cy2}\n" )
 
 
+                #------------------------cross here: before zoom--------------------------------------
+                #print(f" ... cross == {cross}")
+                if not (cross is None):
+                    if senh.add_frame(frame):
+                        w,h,c = frame.shape
+                        # print(w-centerY1, h-centerX1)
+                        #print(f" ... cross {w} {h} /2")
+                        senh.crosson( cross[0], cross[1] , color = "r" ) # dx dy
+                        # senh.setbox(f"CROS",  senh.jpg)
+                        frame = senh.get_frame(  )
+                    else:
+                        print("X... senh did not accept frame (in cross)")
+
                 #=========================== ZOOM ME and OTHERS =======
 
                 if zoomme>1: # FUNNY - it continues to zoom where the mouse pointer is !!!!
@@ -678,6 +657,7 @@ def display2(videodev, save = False,
                         senh.zoom( zoomme , int(centerX1-h/2), int(centerY1-w/2)  )
                         frame = senh.get_frame(  )
 
+                #---------------------------------------------------- rotate ------------------
                 # if rotate180:
                 #     if senh.add_frame(frame):
                 #         senh.setbox(f"ROT",  senh.rot)
@@ -726,6 +706,7 @@ def display2(videodev, save = False,
                     # b = 1m / math.tan( alpha )
 
                     def get_order(dist = 1.7): #  determine order that fits
+                        # list of marks on the ruler
                         wide=0.01
                         while True:
                             wide*=10
@@ -761,10 +742,11 @@ def display2(videodev, save = False,
                             row.append( in0 )
                             row.append( in1 )
                             #row.append( in2 )
-                        return row
+                        return row # Big to small
 
 
-                    def one_mark( dist = 1.7, wide=[1,2] ):
+                    def one_mark( dist = 1.7, wide=[1,2] , speed=0):
+                        #  wide ...  # Big to small
                         # global measure_fov
                         #h,w = frame.shape[0], frame.shape[1]
                         # pixel distance of halfwidth
@@ -781,16 +763,27 @@ def display2(videodev, save = False,
                             #print(f" {radians_per_pixel}radpp {pixwid}   {wide}m <- {dist} ")
                             step = 0
                             mX,mY = int(w/2),int(h/2)
-                            mY= mY+level*step - 60
+                            if not(cross is None):
+                                mX+=cross[0]
+                                mY+=cross[1]
+                            # here I addd the red cross position
+
+
+                            mY= mY+level*step
+
+
                             yA, yB = mY, mY
 
                             xA = mX
                             xB = mX + int(pixwid)
                             color = (0,255,0) # BGR
+                            color = (55,0,255) # BGR same as the red cross
                             if level==0:
+                                # line
                                 cv2.line(frame,
                                          (int(xA), int(yA)), (int(xB), int(yB)),
                                          color, 1)
+                            # vet bars
                             cv2.line(frame,
                                      (int(xA), int(yA+2)), (int(xA), int(yA-2)),
                                      color, 1)
@@ -799,35 +792,85 @@ def display2(videodev, save = False,
                                      color, 1)
 
 
-                            unit = "m"
-                            if level>0: unit=""
-                            unit2 = "m"
 
-                            if iwide<1:
-                                if zoomme==1:
-                                    iwide = round(iwide*10)/10
-                                else:
-                                    iwide = round(iwide*100)/100
+                            unit = "m"
+                            # --- check the biggest to set the unit
+                            if wide[0]<=0.01:
+                                iwide = round(iwide*100*1000)/1000
+                                unit = "cm"
+                            elif wide[0]<=0.1:
+                                iwide = round(iwide*100*100)/100
+                                unit = "cm"
+                            elif wide[0]<1:
+                                iwide = round(iwide*100)/100
+                            else:
+                                iwide = round(iwide*10)/10
+
+                            # only properly round whatever unit
+                            if iwide<=0.01:
+                                iwide = round(iwide*1000)/1000
+                            elif iwide<=0.1:
+                                iwide = round(iwide*100)/100
+                            elif iwide<1:
+                                iwide = round(iwide*10)/10
                             else:
                                 iwide = round(iwide)
 
+                            if level>0: unit="" # no unit during the scale
+                            unit2 = "m"
+
+
 
                             if str(iwide)[:2] == "0.":iwide=str(iwide)[1:]
 
+                            # width on scale
                             cv2.putText(frame, f"{iwide}{unit}",
                                         (int(xB-10), int(  mY -7 )),  # little rightx a bit up y
                                         cv2.FONT_HERSHEY_SIMPLEX, 0.35, color, 1)
                             if level >= 0:
+                                # distance - only at first mark
                                 cv2.putText(frame, f"  at {dist} {unit2}",
-                                            (int(xA-90), int(  mY+5 )),  # little rightx a bit up y
+                                            (int(xA-130), int(  mY+5 )),  # little rightx a bit up y
                                             cv2.FONT_HERSHEY_SIMPLEX, 0.35, color, 1)
                                 cv2.putText(frame, f"  FOV {measure_fov:.1f}deg",
-                                            (int(xA-90), int(  mY+15 )),  # little rightx a bit up y
+                                            (int(xA-130), int(  mY+15 )),  # little rightx a bit up y
+                                            cv2.FONT_HERSHEY_SIMPLEX, 0.35, color, 1)
+                                # I add velocity
+                                if (level==0) and not(tracker1 is None):
+                                    cv2.putText(frame, f"speed {speed:6.2f}m/s",
+                                            (int(xB-50), int(  mY+15 )),  # little rightx a bit up y
                                             cv2.FONT_HERSHEY_SIMPLEX, 0.35, color, 1)
                             level+=1
 
+
+                    # main part of the ruler making---------------
                     order = get_order(dist=measure)
-                    one_mark( dist=measure ,    wide=order )
+                    # speed computation
+                    if not(tracker1 is None) and len(tracker_list)>2:
+                        b=tracker_list[-1]
+                        try:
+                            v_from = -2
+                            a=tracker_list[ v_from]
+                            while True:
+                                dt=(b[2]-a[2]).total_seconds()
+                                if dt>1.0:
+                                    break
+                                else:
+                                    v_from-=1
+                                if len(tracker_list)<abs(v_from): break
+                                a=tracker_list[ v_from]
+                        except:
+                            dt=100000 # velocity 0
+                        c= ( (b[0]-a[0])**2 + (b[1]-a[1])**2 )**0.5
+                        v = c/dt * radians_per_pixel2
+                        v = round( 100* math.tan(v) * measure )/100
+                        # print(f"i... speed = {v} m/s  {dt:.2}==dt " )
+                    else:
+                        v = 0
+                    # plot ruler
+                    one_mark( dist=measure ,    wide=order ,  speed=v)
+
+
 
 
 
@@ -835,6 +878,13 @@ def display2(videodev, save = False,
                 #========================================================
 
                 cv2.imshow(rpi_name, frame ) # 1 window for each RPi
+                # this may be useful?
+                if False:
+                    if not (cropped is None):
+                        cropped = cv2.resize(cropped, (640,480) )
+                        cv2.imshow("tracking1", cropped ) # ZOOM ON TRACK
+                        # cv2.resizeWindow(
+
                 if expande >1 :
                     cv2.resizeWindow( rpi_name, int(expande*frame.shape[1]), int(expande*frame.shape[0]) )
                     #print(frame.shape)
@@ -846,28 +896,39 @@ def display2(videodev, save = False,
 
 
                 # print(f"{centerX1} {centerY1}")
-
-                if (not frame is None) and (rpi_name!="") and (key == ord('h')):
-                    print("h PRESSED! - ")
-                    print("""
+                if cross is None:
+                    if (not frame is None) and (rpi_name!="") and (key == ord('h')):
+                        print("h PRESSED! - ")
+                        print("""
  h ... show this help
 
  a ... SAVE AVI
  A ... stopping AVI
+ s ... saving JPG
+
  z ... cycle zoom 2x (mouse click fixes the center)
  Z ... zoom 1x
  x ... expand 2x ... buggy
 
- s ... saving JPG
  r ... rotate by 180 degrees
 
+ c ... cross on/off
+ hjkl ... move the cross
+ t ... tracker 1 (cross sync, speed)
+ T ... tracker 2 (more steady, but fragile)
+ u ... UNtrack
+
+ m ... measure distance, inc dist  (-v 110,1.2)
+ n ... distance decrease
+ f/F ... inc/dec Field Of View (-v FOV,dist)
+ M ... UNmeasure
+
  w ... openning web browser
  q ... quit
 """)
 
-                if (not frame is None) and (rpi_name!="") and (key == ord('w')):
-                    print("w PRESSED! - openning web browser")
-                    webbrowser.open( videodev.replace("/video","" ) ) # BRUTAL
+
+                #-----------------------------------------------------rotate zoom
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('r')):
                     print("r PRESSED! - rotate change")
@@ -877,29 +938,33 @@ def display2(videodev, save = False,
                         rotate = 180
 
 
-                if (not frame is None) and (rpi_name!="") and (key == ord('a')):
-                    print("a PRESSED! - saving AVI")
-                    save = True
-                    sfilenamea,saviout = setupsave()
-                    print(">>>", sfilenamea )
-
-                if (not frame is None) and (rpi_name!="") and (key == ord('A')):
-                    print("A PRESSED! - STOPPING stopping saving AVI")
-                    save = False
-                    sfilenamea = ""
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('z')):
-                    print("z PRESSED! - ZOOM 2")
+                    print(f"z PRESSED! - ZOOM {zoomme}x")
                     zoomme*= 2
                     if zoomme>16:
                         zoomme=1
                     sfilenamea = ""
 
+
                 if (not frame is None) and (rpi_name!="") and (key == ord('Z')):
                     print("Z PRESSED! - ZOOM ended")
                     zoomme = 1
                     sfilenamea = ""
 
+                # ------------------------------------------ web  pause quit expa---------------
+
+
+                if (not frame is None) and (rpi_name!="") and (key == ord('w')):
+                    print("w PRESSED! - openning web browser")
+                    webbrowser.open( videodev.replace("/video","" ) ) # BRUTAL
+
+
+                if (not frame is None) and (rpi_name!="") and (key == ord(' ')):
+                    print("SPC PRESSED! - pause/play")
+                    pause = not pause
+                    print(f" ... pause = {pause}" )
+
                 if (not frame is None) and (rpi_name!="") and (key == ord('q')):
                     print("q PRESSED!")
                     sys.exit(1)
@@ -914,13 +979,26 @@ def display2(videodev, save = False,
                     else:
                         expande = 2
 
+                #-----------------------------------------------------save s a
+
+
+                if (not frame is None) and (rpi_name!="") and (key == ord('a')):
+                    print("a PRESSED! - saving AVI")
+                    save = True
+                    sfilenamea,saviout = setupsave()
+                    print(">>>", sfilenamea )
+
+                if (not frame is None) and (rpi_name!="") and (key == ord('A')):
+                    print("A PRESSED! - STOPPING stopping saving AVI")
+                    save = False
+                    sfilenamea = ""
 
                 saved_jpg = False
                 if (not frame is None) and (rpi_name!="") and (key == ord('s')):
                     print("s PRESSED!")
                     sname = "snapshot"
                     saved_jpg = True
-                    sfilename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
+                    sfilename = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
                     # defined above # sme = socket.gethostname()
                     sfilename = f"{sme}{sname}_{sfilename}.jpg"
                     sfilename = os.path.expanduser( "~/"+sfilename )
@@ -931,37 +1009,82 @@ def display2(videodev, save = False,
                         print('Image is successfully saved as file.', sfilename)
 
 
+                #------------------------- measure and cross -------------------------------
+
+                if (not frame is None) and (rpi_name!=""):
+                    if (key == ord('c')):
+                        if cross is None:
+                            cross = [ 0,0 ]
+                            print(f"c PRESSED! - cross = {cross}")
+                        else:
+                            cross = None
+                if not(cross is None):
+                    if (not frame is None) and (rpi_name!="") and (key == ord('h')): #<
+                        cross[0]-=5
+                    if (not frame is None) and (rpi_name!="") and (key == ord('j')): #v
+                        cross[1]+=5
+                    if (not frame is None) and (rpi_name!="") and (key == ord('k')): #^
+                        cross[1]-=5
+                    if (not frame is None) and (rpi_name!="") and (key == ord('l')): #>
+                        cross[0]+=5
+
+
+
                 if (not frame is None) and (rpi_name!="") and (key == ord('n')):
-                    print("m PRESSED! - measure far")
+                    print("n PRESSED! - measure distance - far")
                     measure = round(10*measure/1.15)/10
                     if measure <1:
                         measure = 1
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('m')):
-                    print("n PRESSED! - measure close")
+                    print("m PRESSED! - measure distance - closer")
                     if measure == 0:
                         measure = 1
                     measure = round(10*measure*1.15)/10
-                    if measure >10000:
-                        measure = 10000
+                    if measure >20000:
+                        measure = 20000
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('M')):
                     print("M PRESSED! - DEmeasure")
                     measure = 0
 
+                if (not frame is None) and (rpi_name!="") and (key == ord('f')):
+                    print("f PRESSED! - FOV increase")
+                    measure_fov = measure_fov*1.25
+                    if measure_fov>3:
+                        measure_fov = round(measure_fov)
+                    else:
+                        measure_fov = round(measure_fov*10)/10
+                    if measure_fov>180:
+                        measure_fov = 180
+                if (not frame is None) and (rpi_name!="") and (key == ord('F')):
+                    print("F PRESSED! - FOV decrease")
+                    measure_fov = measure_fov/1.25
+                    if measure_fov>3:
+                        measure_fov = round(measure_fov)
+                    else:
+                        measure_fov = round(measure_fov*10)/10
+
+                    if measure_fov<0.3:
+                        measure_fov=0.3
+
+
+
+
+                #---------------- trackers-------------------------------------------
 
-                #---------------- trackers--------------
                 if (not frame is None) and (rpi_name!="") and (key == ord('t')):
-                    print("t PRESSED! - track" ,tracker)
-                    tracker = cv2.TrackerCSRT_create() # KCF GOTURN MIL
+                    print("t PRESSED! - track" ,tracker1)
+                    tracker1 = cv2.TrackerCSRT_create() # KCF GOTURN MIL
                     bbox = cv2.selectROI(frame)
                     if (len(bbox)<4)or(bbox[-1]<10):
-                        tracker = None
+                        tracker1 = None
                         print("i... fail init track")
                     else:
                         #bbox = tuple([ i+0.5 for i in bbox ])
-                        ok = tracker.init(frame,bbox)
-
+                        ok = tracker1.init(frame,bbox)
+                        tracker_list=[]
+                        tracker1_fname = datetime.datetime.now().strftime("tracker1_%Y%m%d_%H%M%S.dat")
                         # #------------this is for histotracker
                         # x,y,w,h = bbox
                         # track_window = bbox
@@ -976,7 +1099,7 @@ def display2(videodev, save = False,
                         # orb = cv2.ORB_create() # I dont use now
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('T')):
-                    print("t PRESSED! - track" ,tracker)
+                    print("t PRESSED! - track" ,tracker2)
                     tracker2 = cv2.TrackerKCF_create() # KCF GOTURN MIL
                     bbox2 = cv2.selectROI(frame)
                     if (len(bbox2)<4) or  (bbox2[-1]<10):
@@ -985,11 +1108,14 @@ def display2(videodev, save = False,
                     else:
                         #bbox2 = tuple([ i+0.5 for i in bbox2 ])
                         ok2 = tracker2.init(frame,bbox2)
+                        tracker2_fname = datetime.datetime.now().strftime("tracker2_%Y%m%d_%H%M%S.dat")
+
 
                 if (not frame is None) and (rpi_name!="") and (key == ord('u')):
                     print("u PRESSED! - UNtrack" )
-                    tracker = None
+                    tracker1 = None
                     tracker2 = None
+                    tracker_date = None
 
 
 
