@@ -289,7 +289,7 @@ def get_gem( cc, capa ):
         maxm = cc.getmax_gamma()
         gm10 = (gm-minm)/(maxm-minm)
         agm = (gm,gmd,minm,maxm,gm10)
-        print(f"I... gamma             {gm:4d};  range({minm:4d},{maxm:5d}) def: {gmd} {gm10:.5f}")
+        print(f"I... gamma             {gm:5d};  range({minm:4d},{maxm:5d}) def: {gmd} {gm10:.5f}")
 
     if "exposure_absolute" in capa:
         ex = cc.get_exposure_absolute()
@@ -298,7 +298,7 @@ def get_gem( cc, capa ):
         maxe = cc.getmax_exposure_absolute()
         ex10 = (ex-mine)/(maxe-mine)
         aex = (ex,exd,mine,maxe,ex10)
-        print(f"I... exposure_absolute {ex:4d};  range({mine:4d},{maxe:5d}) def: {exd} {ex10:.5f}")
+        print(f"I... exposure_absolute {ex:5d};  range({mine:4d},{maxe:5d}) def: {exd} {ex10:.5f}")
 
     if "exposure_auto" in capa:
         ea = cc.get_exposure_auto()
@@ -306,7 +306,7 @@ def get_gem( cc, capa ):
         minea = cc.getmin_exposure_auto()
         maxea = cc.getmax_exposure_auto()
         aea = (ea,ead,minea,maxea)
-        print(f"I... exposure auto     {ea:4d};  range({minea:4d},{maxea:5d}) def: {ead}")
+        print(f"I... exposure auto     {ea:5d};  range({minea:4d},{maxea:5d}) def: {ead}")
 
     # A STRANGE CAMERA
     if "exposure" in capa:
@@ -348,8 +348,9 @@ def set_gem(cc, gain=None, expo=None, mmaga=None):
 
 
 
-
+    print(f"i1... in set_gem: ex={ex}")
     if not expo is None:
+        print(f"ix... expo demand {expo}")
         if (expo == "auto"):
             if "exposure_absolute" in capa:
                 print("D... AUTO EXPOSURE ON")
@@ -397,6 +398,9 @@ def set_gem(cc, gain=None, expo=None, mmaga=None):
                 cc.set_exposure(ex)
             else:
                 print("X... exposure_absolute NOR exposure not in capacities")
+
+    print(f"i2... in set_gem: ex={ex}")
+
     if not gain is None:
         if gain == "def":
             if "gain" in capa:
@@ -491,7 +495,7 @@ def get_resolutions(vidnum):
     output = [tupl for tupl in {item for item in output }]
     output = sorted( output, key = lambda x: int(x.split("x")[0]) * int(x.split("x")[1]) )
     #o = [ float(o.split("x")[0]+"."+o.split("x")[1]) for o in output]
-    print(output)
+    # print(output)
     return output
 
 
